searchData={"items":[{"type":"module","title":"Snex","doc":"Easy and efficient Python interop for Elixir.","ref":"Snex.html"},{"type":"module","title":"Highlights - Snex","doc":"- 🛡️ **Robust & Isolated:**\n  Run multiple Python interpreters in separate OS processes, preventing GIL issues from affecting your Elixir application.\n- 📦 **Declarative Environments:**\n  Leverages [`uv`][uv] to manage Python versions and dependencies, embedding them into your application's release for consistent deployments.\n- ✨ **Ergonomic Interface:**\n  A powerful and efficient interface with explicit control over data passing between Elixir and Python processes.\n- 🤸 **Flexible:**\n  Supports custom Python environments, `asyncio` code, and integration with external Python projects.\n- ⏩ **Forward Compatibility:**\n  Built on stable foundations, so future versions of Python or Elixir are unlikely to require Snex updates to use — they should work day one!","ref":"Snex.html#module-highlights"},{"type":"module","title":"Quick example - Snex","doc":"```elixir\ndefmodule SnexTest.NumpyInterpreter do\n  use Snex.Interpreter,\n    pyproject_toml: \"\"\"\n    [project]\n    name = \"my-numpy-project\"\n    version = \"0.0.0\"\n    requires-python = \"==3.11.*\"\n    dependencies = [\"numpy>=2\"]\n    \"\"\"\nend\n\niex> {:ok, inp} = SnexTest.NumpyInterpreter.start_link()\niex> {:ok, env} = Snex.make_env(inp)\n...>\niex> Snex.pyeval(env, \"\"\"\n...>   import numpy as np\n...>   matrix = np.fromfunction(lambda i, j: (-1) ** (i + j), (s, s), dtype=int)\n...>   \"\"\", %{\"s\" => 6}, returning: \"np.linalg.norm(matrix)\")\n{:ok, 6.0}\n```","ref":"Snex.html#module-quick-example"},{"type":"module","title":"Installation & Requirements - Snex","doc":"- Elixir `>= 1.18`\n- [uv](https://github.com/astral-sh/uv) -\n  a fast Python package & project manager, used by Snex to create and manage Python environments.\n  It has to be available at compilation time but isn't needed at runtime.\n- Python `>= 3.11` -\n  this is the minimum supported version you can run your scripts with.\n  You don't need to have it installed — Snex will fetch it with `uv`.\n\n```elixir\ndef deps do\n  [\n    {:snex, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"Snex.html#module-installation-requirements"},{"type":"module","title":"Core Concepts & Usage - Snex","doc":"","ref":"Snex.html#module-core-concepts-usage"},{"type":"module","title":"Custom Interpreter - Snex","doc":"You can define your Python project settings using `use Snex.Interpreter` in your module.\n\nSet a required Python version and any dependencies —both the Python binary & the dependencies will\nbe fetched & synced at compile time with [uv][uv], and put into your application's priv directory.\n\n```elixir\ndefmodule SnexTest.NumpyInterpreter do\n  use Snex.Interpreter,\n    pyproject_toml: \"\"\"\n    [project]\n    name = \"my-numpy-project\"\n    version = \"0.0.0\"\n    requires-python = \"==3.11.*\"\n    dependencies = [\"numpy>=2\"]\n    \"\"\"\nend\n```\n\nThe modules using `Snex.Interpreter` have to be `start_link`ed to use.\nEach `Snex.Interpreter` (BEAM) process manages a separate Python (OS) process.\n\n### `Snex.pyeval`\n\nThe main way of interacting with the interpreter process is `Snex.pyeval/4` (and other arities).\nThis is the function that runs Python code, returns data from the interpreter, and more.\n\n```elixir\niex> {:ok, interpreter} = SnexTest.NumpyInterpreter.start_link()\niex> {:ok, %Snex.Env{} = env} = Snex.make_env(interpreter)\n...>\niex> Snex.pyeval(env, \"\"\"\n...>   import numpy as np\n...>   matrix = np.fromfunction(lambda i, j: (-1) ** (i + j), (6, 6), dtype=int)\n...>   scalar = np.linalg.norm(matrix)\n...>   \"\"\", returning: \"scalar\")\n{:ok, 6.0}\n```\n\nThe `:returning` option can take any valid Python expression, or an Elixir list of them:\n\n```elixir\niex> {:ok, interpreter} = SnexTest.NumpyInterpreter.start_link()\niex> {:ok, env} = Snex.make_env(interpreter)\n...>\niex> Snex.pyeval(env, \"x = 3\", returning: [\"x\", \"x*2\", \"x**2\"])\n{:ok, [3, 6, 9]}\n```","ref":"Snex.html#module-custom-interpreter"},{"type":"module","title":"Environments - Snex","doc":"`Snex.Env` struct, also called \"environment\", is an Elixir-side reference to Python-side variable context in which your Python code will run.\nNew environments can be allocated with `Snex.make_env/3` (and other arities).\n\nEnvironments are mutable, and will be modified by your Python code.\nIn Python parlance, they are **global & local symbol table** your Python code is executed with.\n\n> #### Important {: .info}\n>\n> **Environments are garbage collected**  \n> When a `%Snex.Env{}` value is cleaned up by the BEAM VM, the Python process is signalled to deallocate the environment associated with that value.\n\nReusing a single environment, you can use variables defined in the previous `Snex.pyeval/4` calls:\n\n```elixir\niex> {:ok, inp} = SnexTest.NumpyInterpreter.start_link()\niex> {:ok, env} = Snex.make_env(inp)\n...>\n...> # `pyeval` does not return a value if not given a `returning` opt\niex> :ok = Snex.pyeval(env, \"x = 10\")\n...>\n...> # additional data can be provided for `pyeval` to put in the environment\n...> # before running the code\niex> :ok = Snex.pyeval(env, \"y = x * z\", %{\"z\" => 2})\n...>\n...> # `pyeval` can also be called with `:returning` opt alone\niex> Snex.pyeval(env, returning: [\"x\", \"y\", \"z\"])\n{:ok, [10, 20, 2]}\n```\n\nUsing `Snex.make_env/2` and `Snex.make_env/3`, you can also create a new environment:\n\n- **copying variables from an old environment**\n  ```elixir\n  Snex.make_env(interpreter, from: old_env)\n  ```\n- **copying variables from multiple environments (later override previous)**\n  ```elixir\n  Snex.make_env(interpreter, from: [\n    oldest_env,\n    {older_env, only: [\"pool\"]},\n    {old_env, except: [\"pool\"]}\n  ]))\n  ```\n- **setting some initial variables (taking precedence over variables from `:from`)**\n  ```elixir\n  Snex.make_env(interpreter, %{\"hello\" => 42.0}, from: {old_env, only: [\"world\"]})\n  ```\n\n> #### Warning {: .warning}\n>\n> The environments you copy from have to belong to the same interpreter!","ref":"Snex.html#module-environments"},{"type":"module","title":"JSON serialization - Snex","doc":"User data sent between Python and Elixir is subject to \"standard\" JSON serialization and deserialization using [`JSON`](https://hexdocs.pm/elixir/JSON.html) on the Elixir side and [`json`](https://docs.python.org/3/library/json.html) on the Python side.\nAmong other things, this means that Python tuples will be serialized as arrays, while Elixir atoms and binaries will be serialized as strings.\n\n```elixir\niex> {:ok, inp} = SnexTest.NumpyInterpreter.start_link()\niex> {:ok, env} = Snex.make_env(inp)\n...>\niex> Snex.pyeval(env, \"x = ('hello', y)\", %{\"y\" => :world}, returning: \"x\")\n{:ok, [\"hello\", \"world\"]}\n```","ref":"Snex.html#module-json-serialization"},{"type":"module","title":"Run async code - Snex","doc":"Code ran by Snex lives in an [`asyncio`](https://docs.python.org/3/library/asyncio.html) loop.\nYou can include async functions in your snippets and await them on the top level:\n\n```elixir\niex> {:ok, inp} = SnexTest.NumpyInterpreter.start_link()\niex> {:ok, env} = Snex.make_env(inp)\n...>\niex> Snex.pyeval(env, \"\"\"\n...>   import asyncio\n...>   async def do_thing():\n...>       await asyncio.sleep(0.01)\n...>       return \"hello\"\n...>\n...>   result = await do_thing()\n...>   \"\"\", returning: [\"result\"])\n{:ok, \"hello\"}\n```","ref":"Snex.html#module-run-async-code"},{"type":"module","title":"Run blocking code - Snex","doc":"A good way to run any blocking code is to prepare and use your own thread or process pools:\n\n```elixir\niex> {:ok, inp} = SnexTest.NumpyInterpreter.start_link()\niex> {:ok, pool_env} = Snex.make_env(inp)\n...>\niex> :ok = Snex.pyeval(pool_env, \"\"\"\n...>   import asyncio\n...>   from concurrent.futures import ThreadPoolExecutor\n...>\n...>   pool = ThreadPoolExecutor(max_workers=cnt)\n...>   loop = asyncio.get_running_loop()\n...>   \"\"\", %{\"cnt\" => 5})\n...>\n...> # You can keep the pool environment around and copy it into new ones\niex> {:ok, env} = Snex.make_env(inp, from: {pool_env, only: [\"pool\", \"loop\"]})\n...>\niex> {:ok, \"world!\"} = Snex.pyeval(env, \"\"\"\n...>   def blocking_io():\n...>       return \"world!\"\n...>\n...>   res = await loop.run_in_executor(pool, blocking_io)\n...>   \"\"\", returning: \"res\")\n{:ok, \"world!\"}\n```","ref":"Snex.html#module-run-blocking-code"},{"type":"module","title":"Use your in-repo project - Snex","doc":"You can reference your existing project path in `use Snex.Interpreter`.\n\nThe existing `pyproject.toml` and `uv.lock` will be used to seed the Python environment.\n\n```elixir\ndefmodule SnexTest.MyProject do\n  use Snex.Interpreter,\n    project_path: \"test/my_python_proj\"\nend\n\n# $ cat test/my_python_proj/foo.py\n# def bar():\n#     return \"hi from bar\"\n\n# Provide the project's path at runtime - you'll likely want to use\n# :code.priv_dir(:your_otp_app) and construct a path relative to that.\niex> {:ok, inp} = SnexTest.MyProject.start_link(environment: %{\n...>   \"PYTHONPATH\" => \"test/my_python_proj\"\n...> })\niex> {:ok, env} = Snex.make_env(inp)\n...>\niex> Snex.pyeval(env, \"import foo\", returning: \"foo.bar()\")\n{:ok, \"hi from bar\"}\n```\n\n[uv]: https://github.com/astral-sh/uv","ref":"Snex.html#module-use-your-in-repo-project"},{"type":"type","title":"Snex.additional_vars/0","doc":"A map of additional variables to be added to the environment.\n\nSee `Snex.make_env/3`.","ref":"Snex.html#t:additional_vars/0"},{"type":"type","title":"Snex.code/0","doc":"A string of Python code to be evaluated.\n\nSee `Snex.pyeval/4`.","ref":"Snex.html#t:code/0"},{"type":"opaque","title":"Snex.env/0","doc":"An \"environment\" is an Elixir-side reference to Python-side variable context in which your Python\ncode will run.\n\nSee `Snex.make_env/3` for more information.","ref":"Snex.html#t:env/0"},{"type":"type","title":"Snex.from_env/0","doc":"A single environment or a list of environments to copy variables from.\n\nSee `Snex.make_env/3`.","ref":"Snex.html#t:from_env/0"},{"type":"function","title":"Snex.make_env/1","doc":"Shorthand for `Snex.make_env/3`:\n\n    Snex.make_env(interpreter, %{} = _additional_vars, [] = _opts)","ref":"Snex.html#make_env/1"},{"type":"function","title":"Snex.make_env/2","doc":"Shorthand for `Snex.make_env/3`:\n\n    # when given a map of `additional_vars`:\n    Snex.make_env(interpreter, additional_vars, [] = _opts)\n\n    # when given an `opts` list:\n    Snex.make_env(interpreter, %{} = _additional_vars, opts)","ref":"Snex.html#make_env/2"},{"type":"function","title":"Snex.make_env/3","doc":"Creates a new environment, `%Snex.Env{}`.\n\nA `%Snex.Env{}` instance is an Elixir-side reference to a variable context in Python.\nThe variable contexts are the **global & local symbol table** the Python code will be executed\nwith using the `Snex.pyeval/2` function.\n\n`additional_vars` are additional variables that will be added to the environment.\nThey will be applied after copying variables from the environments listed in the `:from` option.\n\nReturns a tuple `{:ok, %Snex.Env{}}` on success.","ref":"Snex.html#make_env/3"},{"type":"function","title":"Options - Snex.make_env/3","doc":"* `:from` - a list of environments to copy variables from.\n    Each value in the list can be either a tuple `{%Snex.Env{}, opts}`, or a `%Snex.Env{}`\n    (Shorthand for `{%Snex.Env{}, []}`).\n\n    The following mutually exclusive options are supported:\n\n    * `:only` - a list of variable names to copy from the `from` environment.\n    * `:except` - a list of variable names to exclude from the `from` environment.","ref":"Snex.html#make_env/3-options"},{"type":"function","title":"Examples - Snex.make_env/3","doc":"# Create a new empty environment\n    Snex.make_env(interpreter)\n\n    # Create a new environment with additional variables\n    Snex.make_env(interpreter, %{\"x\" => 1, \"y\" => 2})\n\n    # Create a new environment copying variables from existing environments\n    Snex.make_env(interpreter, from: env)\n    Snex.make_env(interpreter, from: {env, except: [\"y\"]})\n    Snex.make_env(interpreter, from: [env1, {env2, only: [\"x\"]}])\n\n    # Create a new environment with both additional variables and `:from`\n    Snex.make_env(interpreter, %{\"x\" => 1, \"y\" => 2}, from: env)","ref":"Snex.html#make_env/3-examples"},{"type":"type","title":"Snex.make_env_opt/0","doc":"Option for `Snex.make_env/3`.","ref":"Snex.html#t:make_env_opt/0"},{"type":"function","title":"Snex.pyeval/2","doc":"Shorthand for `Snex.pyeval/4`:\n\n    # when given a `code` string:\n    Snex.pyeval(env, code, %{} = _additional_vars, [] = _opts)\n\n    # when given an `additional_vars` map:\n    Snex.pyeval(env, nil = _code, additional_vars, [] = _opts)\n\n    # when given an `opts` list:\n    Snex.pyeval(env, nil = _code, %{} = _additional_vars, opts)","ref":"Snex.html#pyeval/2"},{"type":"function","title":"Snex.pyeval/3","doc":"Shorthand for `Snex.pyeval/4`:\n\n    # when given an `additional_vars` map:\n    Snex.pyeval(env, code, additional_vars, [] = _opts)\n\n    # when given an `opts` list:\n    Snex.pyeval(env, code, %{} = _additional_vars, opts)","ref":"Snex.html#pyeval/3"},{"type":"function","title":"Snex.pyeval/4","doc":"Evaluates a Python code string in the given environment.\n\n`additional_vars` are added to the environment before the code is executed.\nSee `Snex.make_env/3` for more information.\n\nReturns `:ok` on success, or a tuple `{:ok, result}` if `:returning` option is provided.","ref":"Snex.html#pyeval/4"},{"type":"function","title":"Options - Snex.pyeval/4","doc":"* `:returning` - a Python expression or a list of Python expressions to evaluate and return from\n    this function. If not provided, the result will be `:ok`.\n\n  * `:timeout` - the timeout for the evaluation. Can be a `timeout()` or `:infinity`.\n    Default: 5 seconds.","ref":"Snex.html#pyeval/4-options"},{"type":"function","title":"Examples - Snex.pyeval/4","doc":"Snex.pyeval(env, \"\"\"\n      res = [x for x in range(num_range)]\n      \"\"\", %{\"num_range\" => 6}, returning: \"[x * x for x in res]\")\n\n    [0, 1, 4, 9, 16, 25]","ref":"Snex.html#pyeval/4-examples"},{"type":"type","title":"Snex.pyeval_opt/0","doc":"Option for `Snex.pyeval/4`.","ref":"Snex.html#t:pyeval_opt/0"},{"type":"module","title":"Snex.Env","doc":"An environment (`%Snex.Env{}`) is a Elixir-side reference to Python-side variable context in which\nthe Python code is executed.\n\nThis module is not intended to be used directly.\nInstead, you would use the `Snex.make_env/3` to create environments and `Snex.pyeval/4`\nto use them.\n\nSee `m:Snex#module-environments` module documentation for more details.","ref":"Snex.Env.html"},{"type":"module","title":"Snex.Interpreter","doc":"Runs a Python interpreter in a separate OS process.\n\nThis module is responsible for facilitating in-and-out communication between Elixir\nand the spawned Python interpreter.\n\nUsually you won't interact with this module directly.\nInstead, you would create a custom interpreter module with `use Snex.Interpreter`:\n\n    defmodule SnexTest.NumpyInterpreter do\n      use Snex.Interpreter,\n        pyproject_toml: \"\"\"\n        [project]\n        name = \"my-numpy-project\"\n        version = \"0.0.0\"\n        requires-python = \"==3.11.*\"\n        dependencies = [\"numpy>=2\"]\n        \"\"\"\n      end\n\nSee the `m:Snex#module-custom-interpreter` module documentation for more detail.","ref":"Snex.Interpreter.html"},{"type":"function","title":"Snex.Interpreter.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Snex.Interpreter.html#child_spec/1"},{"type":"type","title":"Snex.Interpreter.option/0","doc":"Options for `start_link/1`.","ref":"Snex.Interpreter.html#t:option/0"},{"type":"type","title":"Snex.Interpreter.server/0","doc":"Running instance of `Snex.Interpreter`.","ref":"Snex.Interpreter.html#t:server/0"},{"type":"function","title":"Snex.Interpreter.start_link/1","doc":"Starts a new Python interpreter.\n\nThe interpreter can be used by functions in the `Snex` module.","ref":"Snex.Interpreter.html#start_link/1"},{"type":"function","title":"Options - Snex.Interpreter.start_link/1","doc":"* `:python` - The Python executable to use. This can be a full path or a command to     find via `System.find_executable/1`.\n  * `:environment` - A map of environment variables to set when running the Python     executable.\n  * any other options will be passed to `GenServer.start_link/3`.","ref":"Snex.Interpreter.html#start_link/1-options"},{"type":"exception","title":"Snex.Error","doc":"Domain-specific errors returned by `Snex`.","ref":"Snex.Error.html"},{"type":"type","title":"Snex.Error.env_key_not_found/0","doc":"The error code indicating a key is not found in the `t:Snex.env/0`.","ref":"Snex.Error.html#t:env_key_not_found/0"},{"type":"type","title":"Snex.Error.env_not_found/0","doc":"The error code indicating an environment referenced by the passed in `t:Snex.env/0` is not found in\nthe Python interpreter that ran the command.","ref":"Snex.Error.html#t:env_not_found/0"},{"type":"type","title":"Snex.Error.error_code/0","doc":"Error codes for all errors.","ref":"Snex.Error.html#t:error_code/0"},{"type":"type","title":"Snex.Error.internal_error/0","doc":"The error code for an internal error of Snex.","ref":"Snex.Error.html#t:internal_error/0"},{"type":"type","title":"Snex.Error.python_runtime_error/0","doc":"The error code for a runtime error.","ref":"Snex.Error.html#t:python_runtime_error/0"},{"type":"type","title":"Snex.Error.t/0","doc":"The type of `Snex.Error`.","ref":"Snex.Error.html#t:t/0"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS","ref":"license.html"},{"type":"extras","title":"NOTICE","doc":"LEGAL NOTICE INFORMATION\n------------------------\n\nAll the files in this distribution are copyright to the terms below.\n\nCopyright 2025 Konrad Zemek  \n\nLicensed under the Apache License, Version 2.0 (the \"License\");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at\n\n    https://www.apache.org/licenses/LICENSE-2.0\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language","ref":"notice.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.1"}}